@@ -13,117 +13,197 @@
 # limitations under the License.
 from __future__ import unicode_literals
 
-import os
-import textwrap
-
 import pretend
 import pytest
+from requests.exceptions import HTTPError
 
 from twine.commands import upload
-from twine import package, cli, exceptions, settings
+from twine import package, cli, exceptions
 import twine
 
 import helpers
 
+SDIST_FIXTURE = 'tests/fixtures/twine-1.5.0.tar.gz'
 WHEEL_FIXTURE = 'tests/fixtures/twine-1.5.0-py2.py3-none-any.whl'
+RELEASE_URL = 'https://pypi.org/project/twine/1.5.0/'
+NEW_SDIST_FIXTURE = 'tests/fixtures/twine-1.6.5.tar.gz'
+NEW_WHEEL_FIXTURE = 'tests/fixtures/twine-1.6.5-py2.py3-none-any.whl'
+NEW_RELEASE_URL = 'https://pypi.org/project/twine/1.6.5/'
 
 
-def test_successful_upload(tmpdir):
-    pypirc = os.path.join(str(tmpdir), ".pypirc")
-    dists = ["tests/fixtures/twine-1.5.0-py2.py3-none-any.whl"]
-
-    with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
-            [pypi]
-            username:foo
-            password:bar
-        """))
-
-    upload_settings = settings.Settings(
-        repository_name="pypi", sign=None, identity=None, username=None,
-        password=None, comment=None, cert=None, client_cert=None,
-        sign_with=None, config_file=pypirc, skip_existing=False,
-        repository_url=None, verbose=False,
-    )
+def test_successful_upload(make_settings, capsys):
+    upload_settings = make_settings()
 
     stub_response = pretend.stub(
-        is_redirect=False, status_code=201, raise_for_status=lambda: None
+        is_redirect=False,
+        status_code=201,
+        raise_for_status=lambda: None
     )
+
     stub_repository = pretend.stub(
-        upload=lambda package: stub_response, close=lambda: None
+        upload=lambda package: stub_response,
+        close=lambda: None,
+        release_urls=lambda packages: {RELEASE_URL, NEW_RELEASE_URL}
     )
 
     upload_settings.create_repository = lambda: stub_repository
 
-    result = upload.upload(upload_settings, dists)
+    result = upload.upload(upload_settings, [
+        WHEEL_FIXTURE, SDIST_FIXTURE, NEW_SDIST_FIXTURE, NEW_WHEEL_FIXTURE
+    ])
 
-    # Raising an exception or returning anything truthy would mean that the
-    # upload has failed
+    # A truthy result means the upload failed
     assert result is None
 
+    captured = capsys.readouterr()
+    assert captured.out.count(RELEASE_URL) == 1
+    assert captured.out.count(NEW_RELEASE_URL) == 1
 
-def test_get_config_old_format(tmpdir):
-    pypirc = os.path.join(str(tmpdir), ".pypirc")
 
-    with open(pypirc, "w") as fp:
-        fp.write(textwrap.dedent("""
+@pytest.mark.parametrize('verbose', [False, True])
+def test_exception_for_http_status(verbose, make_settings, capsys):
+    upload_settings = make_settings()
+    upload_settings.verbose = verbose
+
+    stub_response = pretend.stub(
+        is_redirect=False,
+        status_code=403,
+        text="Invalid or non-existent authentication information",
+        raise_for_status=pretend.raiser(HTTPError)
+    )
+
+    stub_repository = pretend.stub(
+        upload=lambda package: stub_response,
+        close=lambda: None,
+    )
+
+    upload_settings.create_repository = lambda: stub_repository
+
+    with pytest.raises(HTTPError):
+        upload.upload(upload_settings, [WHEEL_FIXTURE])
+
+    captured = capsys.readouterr()
+    assert RELEASE_URL not in captured.out
+
+    if verbose:
+        assert stub_response.text in captured.out
+        assert '--verbose' not in captured.out
+    else:
+        assert stub_response.text not in captured.out
+        assert '--verbose' in captured.out
+
+
+def test_get_config_old_format(make_settings, pypirc):
+    try:
+        make_settings("""
             [server-login]
             username:foo
             password:bar
-        """))
-
-    try:
-        settings.Settings(
-            repository_name="pypi", sign=None, identity=None, username=None,
-            password=None, comment=None, cert=None, client_cert=None,
-            sign_with=None, config_file=pypirc, skip_existing=False,
-            repository_url=None, verbose=False,
-        )
+        """)
     except KeyError as err:
-        assert err.args[0] == (
-            "Missing 'pypi' section from the configuration file\n"
-            "or not a complete URL in --repository-url.\n"
-            "Maybe you have a out-dated '{0}' format?\n"
-            "more info: "
-            "https://docs.python.org/distutils/packageindex.html#pypirc\n"
-        ).format(pypirc)
+        assert all(text in err.args[0] for text in [
+            "'pypi'",
+            "--repository-url",
+            pypirc,
+            "https://docs.python.org/",
+        ])
 
 
-def test_deprecated_repo(tmpdir):
+def test_deprecated_repo(make_settings):
     with pytest.raises(exceptions.UploadToDeprecatedPyPIDetected) as err:
-        pypirc = os.path.join(str(tmpdir), ".pypirc")
-        dists = ["tests/fixtures/twine-1.5.0-py2.py3-none-any.whl"]
+        upload_settings = make_settings("""
+            [pypi]
+            repository: https://pypi.python.org/pypi/
+            username:foo
+            password:bar
+        """)
 
-        with open(pypirc, "w") as fp:
-            fp.write(textwrap.dedent("""
-                [pypi]
-                repository: https://pypi.python.org/pypi/
-                username:foo
-                password:bar
-            """))
+        upload.upload(upload_settings, [WHEEL_FIXTURE])
 
-        upload_settings = settings.Settings(
-            repository_name="pypi", sign=None, identity=None, username=None,
-            password=None, comment=None, cert=None, client_cert=None,
-            sign_with=None, config_file=pypirc, skip_existing=False,
-            repository_url=None, verbose=False,
-        )
+    assert all(text in err.value.args[0] for text in [
+        "https://pypi.python.org/pypi/",
+        "https://upload.pypi.org/legacy/",
+        "https://test.pypi.org/legacy/",
+        "https://packaging.python.org/",
+    ])
 
-        upload.upload(upload_settings, dists)
 
-    assert err.value.args[0] == (
-        "You're trying to upload to the legacy PyPI site "
-        "'https://pypi.python.org/pypi/'. "
-        "Uploading to those sites is deprecated. \n "
-        "The new sites are pypi.org and test.pypi.org. Try using "
-        "https://upload.pypi.org/legacy/ "
-        "(or https://test.pypi.org/legacy/) "
-        "to upload your packages instead. "
-        "These are the default URLs for Twine now. \n "
-        "More at "
-        "https://packaging.python.org/guides/migrating-to-pypi-org/ ."
+def test_exception_for_redirect(make_settings):
+    upload_settings = make_settings("""
+        [pypi]
+        repository: https://test.pypi.org/legacy
+        username:foo
+        password:bar
+    """)
+
+    stub_response = pretend.stub(
+        is_redirect=True,
+        status_code=301,
+        headers={'location': 'https://test.pypi.org/legacy/'}
     )
 
+    stub_repository = pretend.stub(
+        upload=lambda package: stub_response,
+        close=lambda: None
+    )
+
+    upload_settings.create_repository = lambda: stub_repository
+
+    with pytest.raises(exceptions.RedirectDetected) as err:
+        upload.upload(upload_settings, [WHEEL_FIXTURE])
+
+    assert "https://test.pypi.org/legacy/" in err.value.args[0]
+
+
+def test_prints_skip_message_for_uploaded_package(make_settings, capsys):
+    upload_settings = make_settings(skip_existing=True)
+
+    stub_repository = pretend.stub(
+        # Short-circuit the upload, so no need for a stub response
+        package_is_uploaded=lambda package: True,
+        release_urls=lambda packages: {},
+        close=lambda: None
+    )
+
+    upload_settings.create_repository = lambda: stub_repository
+
+    result = upload.upload(upload_settings, [WHEEL_FIXTURE])
+
+    # A truthy result means the upload failed
+    assert result is None
+
+    captured = capsys.readouterr()
+    assert "Skipping twine-1.5.0-py2.py3-none-any.whl" in captured.out
+    assert RELEASE_URL not in captured.out
+
+
+def test_prints_skip_message_for_response(make_settings, capsys):
+    upload_settings = make_settings(skip_existing=True)
+
+    stub_response = pretend.stub(
+        is_redirect=False,
+        status_code=409,
+    )
+
+    stub_repository = pretend.stub(
+        # Do the upload, triggering the error response
+        package_is_uploaded=lambda package: False,
+        release_urls=lambda packages: {},
+        upload=lambda package: stub_response,
+        close=lambda: None
+    )
+
+    upload_settings.create_repository = lambda: stub_repository
+
+    result = upload.upload(upload_settings, [WHEEL_FIXTURE])
+
+    # A truthy result means the upload failed
+    assert result is None
+
+    captured = capsys.readouterr()
+    assert "Skipping twine-1.5.0-py2.py3-none-any.whl" in captured.out
+    assert RELEASE_URL not in captured.out
+
 
 def test_skip_existing_skips_files_already_on_PyPI(monkeypatch):
     response = pretend.stub(
@@ -193,7 +273,7 @@ def test_skip_upload_respects_skip_existing(monkeypatch):
 
 
 def test_values_from_env(monkeypatch):
-    def none_upload(*args, **kwargs):
+    def none_upload(*args, **settings_kwargs):
         pass
 
     replaced_upload = pretend.call_recorder(none_upload)
